import * as readline from "readline-sync";

export class Tool {
  public toolName: string = "Tool";
  public capacity: number = 100;
  public action(): void {
    this.capacity -= 10;
  }
}

export class Saw extends Tool {
  public override toolName: string = "Saw";
  public override action(): void {
    super.action();
    if (this.capacity !== 0 || this.capacity > 0) {
      console.log("============================================");
      console.log("ZZZZZZZZZZZZ", `Capacity: ${this.capacity}`);
      console.log("============================================");
    }
  }
}
export class Axe extends Tool {
  public override toolName: string = "Axe";
  public override action(): void {
    super.action();
    if (this.capacity !== 0 || this.capacity > 0) {
      console.log("============================================");
      console.log("Тук Тук Тук", `Capacity: ${this.capacity}`);
      console.log("============================================");
    }
  }
}
export class Drill extends Tool {
  public override toolName: string = "Drill";
  public override action(): void {
    super.action();
    if (this.capacity !== 0 || this.capacity > 0) {
      console.log("============================================");
      console.log("DDDDRRRRRRRRRRRRRRR", `Capacity: ${this.capacity}`);
      console.log("============================================");
    }
  }
}
export class Hammer extends Tool {
  public override toolName: string = "Hammer";
  public override action(): void {
    super.action();
    if (this.capacity !== 0 || this.capacity > 0) {
      console.log("============================================");
      console.log("Bam Bam Bam", `Capacity: ${this.capacity}`);
      console.log("============================================");
    }
  }
}
export class Screwdriver extends Tool {
  public override toolName: string = "Screwdriver";
  public override action(): void {
    super.action();
    if (this.capacity !== 0 || this.capacity > 0) {
      console.log("============================================");
      console.log("Scrrrrrrrrrrr", `Capacity: ${this.capacity}`);
      console.log("============================================");
    } else {
    }
  }
}

export class Robot {
  public tool: Tool | null;
  public name: string;
  constructor(name: string = "D2R") {
    this.tool = null;
    this.name = name;
  }
  setup_tool(tool: Tool) {
    this.tool = tool;
  }
  drop_tool() {
    this.tool = null;
    console.log("============================================");
    console.log("Tool has been droped!");
    console.log("============================================");
  }
  action() {
    if (!this.tool) {
      console.log("============================================");
      console.log("I don't have any of tools! :(");
      console.log("============================================");
    } else if (this.tool?.capacity && this.tool.capacity > 0) {
      this.tool.action();
    } else {
      console.log("============================================");
      console.log(
        "Your tool is broken and it does not work properly!\nPlease, drop this tool and get enother one."
      );
      console.log("============================================");
    }
  }
}

type TRobotTools = {
  saw: Tool;
  hammer: Tool;
  screwdriver: Tool;
  axe: Tool;
  drill: Tool;
};

export class Control {
  public robot: Robot;
  public tools: TRobotTools = {
    saw: new Saw(),
    hammer: new Hammer(),
    screwdriver: new Screwdriver(),
    axe: new Axe(),
    drill: new Drill(),
  };
  constructor() {
    this.robot = new Robot();
  }
  start() {
    let out: boolean = true;
    console.log("Hello, your robot name is D2R. Please, take controle of him. \n");
    
    while (out) {
      const answerFromUser: string | null = readline.question(
        `Choose one of the commands: \n1) ${this.robot.tool ? "Use the "+ this.robot.tool.toolName : "No tools to use!"}\n2) ${this.robot.tool ? "Drop the " + this.robot.tool.toolName : "No tools to drop!"}\n3) ${this.robot.tool ? "Your tool is " + this.robot.tool.toolName  : "Choose one of the tools"}\n4) Exit:\n `
      );
      switch (answerFromUser) {
      
        case "1":
          this.robot.action();
          break;
        case "2":
          this.robot.drop_tool();
          break;
        case "3":
          const answer = readline.question(
            "Please, choose one of the tool: \n1) Saw\n2) Hammer\n3) screwdriver\n4) Axe\n5) Drill\n"
          );
          switch (answer) {
            case "1":
              this.robot.setup_tool(this.tools.saw);
              break;
            case "2":
              this.robot.setup_tool(this.tools.hammer);
              break;
            case "3":
              this.robot.setup_tool(this.tools.screwdriver);
              break;
            case "4":
              this.robot.setup_tool(this.tools.axe);
              break;
            case "5":
              this.robot.setup_tool(this.tools.drill);
              break;
            default:
              console.log("============================================");
              console.log("Please, choose correct option");
              console.log("============================================");
              break;
          }
          break;
        case "4":
          out = false;
        default:
          console.log("============================================");
          console.log("Please, choose correct command");
          console.log("============================================");
          break;
      }
    }
  }
}
